export { default as loader } from '@monaco-editor/loader';
import * as vue from 'vue';
import { ExtractPropTypes, PropType } from 'vue';
import * as monacoEditor from 'monaco-editor/esm/vs/editor/editor.api';
import * as monaco_editor from 'monaco-editor';

declare type Nullable<T> = T | null;
declare type MonacoEditor = typeof monacoEditor;

declare const editorProps: {
    defaultValue: StringConstructor;
    defaultPath: StringConstructor;
    defaultLanguage: StringConstructor;
    value: StringConstructor;
    language: StringConstructor;
    path: StringConstructor;
    theme: {
        type: StringConstructor;
        default: string;
    };
    line: NumberConstructor;
    options: {
        type: ObjectConstructor;
        default: () => {};
    };
    overrideServices: {
        type: ObjectConstructor;
        default: () => {};
    };
    saveViewState: {
        type: BooleanConstructor;
        default: boolean;
    };
    width: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    className: StringConstructor;
    'onUpdate:value': PropType<(value: string | undefined) => void>;
    onBeforeMount: PropType<(monaco: MonacoEditor) => void>;
    onMount: PropType<(editor: monacoEditor.editor.IStandaloneCodeEditor, monaco: MonacoEditor) => void>;
    onChange: PropType<(value: string | undefined, event: monacoEditor.editor.IModelContentChangedEvent) => void>;
    onValidate: PropType<(markers: monacoEditor.editor.IMarker[]) => void>;
};
declare type EditorProps = ExtractPropTypes<typeof editorProps>;

declare const _default$1: vue.DefineComponent<{
    defaultValue: StringConstructor;
    defaultPath: StringConstructor;
    defaultLanguage: StringConstructor;
    value: StringConstructor;
    language: StringConstructor;
    path: StringConstructor;
    theme: {
        type: StringConstructor;
        default: string;
    };
    line: NumberConstructor;
    options: {
        type: ObjectConstructor;
        default: () => {};
    };
    overrideServices: {
        type: ObjectConstructor;
        default: () => {};
    };
    saveViewState: {
        type: BooleanConstructor;
        default: boolean;
    };
    width: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    className: StringConstructor;
    'onUpdate:value': vue.PropType<(value: string | undefined) => void>;
    onBeforeMount: vue.PropType<(monaco: typeof monacoEditor) => void>;
    onMount: vue.PropType<(editor: monacoEditor.editor.IStandaloneCodeEditor, monaco: typeof monacoEditor) => void>;
    onChange: vue.PropType<(value: string | undefined, event: monacoEditor.editor.IModelContentChangedEvent) => void>;
    onValidate: vue.PropType<(markers: monacoEditor.editor.IMarker[]) => void>;
}, () => JSX.Element, unknown, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, Record<string, any>, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{
    defaultValue: StringConstructor;
    defaultPath: StringConstructor;
    defaultLanguage: StringConstructor;
    value: StringConstructor;
    language: StringConstructor;
    path: StringConstructor;
    theme: {
        type: StringConstructor;
        default: string;
    };
    line: NumberConstructor;
    options: {
        type: ObjectConstructor;
        default: () => {};
    };
    overrideServices: {
        type: ObjectConstructor;
        default: () => {};
    };
    saveViewState: {
        type: BooleanConstructor;
        default: boolean;
    };
    width: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        default: string;
    };
    className: StringConstructor;
    'onUpdate:value': vue.PropType<(value: string | undefined) => void>;
    onBeforeMount: vue.PropType<(monaco: typeof monacoEditor) => void>;
    onMount: vue.PropType<(editor: monacoEditor.editor.IStandaloneCodeEditor, monaco: typeof monacoEditor) => void>;
    onChange: vue.PropType<(value: string | undefined, event: monacoEditor.editor.IModelContentChangedEvent) => void>;
    onValidate: vue.PropType<(markers: monacoEditor.editor.IMarker[]) => void>;
}>>, {
    width: string | number;
    height: string | number;
    theme: string;
    options: Record<string, any>;
    overrideServices: Record<string, any>;
    saveViewState: boolean;
}>;

declare const _default: vue.DefineComponent<{
    width: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    isEditorReady: {
        type: BooleanConstructor;
        required: true;
    };
    className: StringConstructor;
    setContainerRef: vue.PropType<(ref: HTMLElement | null) => void>;
}, () => JSX.Element, unknown, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, Record<string, any>, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{
    width: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    isEditorReady: {
        type: BooleanConstructor;
        required: true;
    };
    className: StringConstructor;
    setContainerRef: vue.PropType<(ref: HTMLElement | null) => void>;
}>>, {}>;

declare const monacoContainerProps: {
    width: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    height: {
        type: (NumberConstructor | StringConstructor)[];
        required: true;
    };
    isEditorReady: {
        type: BooleanConstructor;
        required: true;
    };
    className: StringConstructor;
    setContainerRef: PropType<(ref: HTMLElement | null) => void>;
};
declare type MonacoContainerProps = ExtractPropTypes<typeof monacoContainerProps>;

declare function useMonaco(): {
    monacoRef: vue.ShallowRef<Nullable<typeof monaco_editor>>;
    unload: () => void;
};

export { EditorProps, _default as MonacoContainer, MonacoContainerProps, _default$1 as default, useMonaco };
