(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@monaco-editor/loader'), require('vue')) :
  typeof define === 'function' && define.amd ? define(['exports', '@monaco-editor/loader', 'vue'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.monaco_vue = {}, global.monaco_loader, global.Vue));
})(this, (function (exports, loader, vue) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var loader__default = /*#__PURE__*/_interopDefaultLegacy(loader);

  function useMonaco() {
    const monacoRef = vue.shallowRef(null); // monaco mount

    const monacoLoader = loader__default["default"].init();

    const unload = () => monacoLoader.cancel();

    monacoLoader.then(monacoInstance => monacoRef.value = monacoInstance).catch(error => {
      if ((error === null || error === void 0 ? void 0 : error.type) !== 'cancelation') {
        console.error('Monaco initialization error:', error);
      }
    });
    return {
      monacoRef,
      unload
    };
  }

  const editorProps = {
    defaultValue: String,
    defaultPath: String,
    defaultLanguage: String,
    value: String,
    language: String,
    path: String,

    /* === */
    theme: {
      type: String,
      default: 'light'
    },
    line: Number,
    options: {
      type: Object,
      default: () => ({})
    },
    overrideServices: {
      type: Object,
      default: () => ({})
    },
    saveViewState: {
      type: Boolean,
      default: true
    },
    // keepCurrentModel: {
    //   type: Boolean,
    //   default: false,
    // },

    /* === */
    width: {
      type: [Number, String],
      default: '100%'
    },
    height: {
      type: [Number, String],
      default: '100%'
    },
    className: String,

    /* === */
    'onUpdate:value': Function,
    onBeforeMount: Function,
    onMount: Function,
    onChange: Function,
    onValidate: Function
  };

  function isUndefined(v) {
    return v === undefined;
  }
  function getOrCreateModel(monaco, value, language, path) {
    return getModel(monaco, path) || createModel(monaco, value, language, path);
  }

  function getModel(monaco, path) {
    return monaco.editor.getModel(createModelUri(monaco, path));
  }

  function createModel(monaco, value, language, path) {
    return monaco.editor.createModel(value, language, path ? createModelUri(monaco, path) : undefined);
  }

  function createModelUri(monaco, path) {
    return monaco.Uri.parse(path);
  }

  const monacoContainerProps = {
    width: {
      type: [Number, String],
      required: true
    },
    height: {
      type: [Number, String],
      required: true
    },
    isEditorReady: {
      type: Boolean,
      required: true
    },
    className: String,
    setContainerRef: Function
  };

  const styles = {
    wrapper: {
      display: 'flex',
      position: 'relative',
      textAlign: 'initial'
    },
    fullWidth: {
      width: '100%'
    },
    hide: {
      display: 'none'
    }
  };

  const loadingStyles = {
    display: 'flex',
    height: '100%',
    width: '100%',
    justifyContent: 'center',
    alignItems: 'center'
  };
  var Loading = vue.defineComponent((props, _ref) => {
    let {
      slots
    } = _ref;
    return () => {
      var _slots$default;

      return vue.createVNode("div", {
        "style": vue.normalizeStyle(loadingStyles)
      }, [(_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)]);
    };
  });

  var MonacoContainer = vue.defineComponent({
    props: monacoContainerProps,

    setup(props, _ref) {
      let {
        slots
      } = _ref;
      const wrapperStyle = vue.computed(() => {
        const {
          width,
          height
        } = props;
        return vue.normalizeStyle({ ...styles.wrapper,
          width,
          height
        });
      });
      const containerStyle = vue.computed(() => {
        return vue.normalizeStyle({ ...styles.fullWidth,
          ...(!props.isEditorReady && styles.hide)
        });
      });
      return () => {
        var _slots$default;

        return vue.createVNode("div", {
          "style": wrapperStyle.value
        }, [!props.isEditorReady && vue.createVNode(Loading, null, {
          default: () => [(_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)]
        }), vue.createVNode("div", {
          "ref": props.setContainerRef,
          "style": containerStyle.value,
          "class": props.className
        }, null)]);
      };
    }

  });

  var Editor = vue.defineComponent({
    props: editorProps,

    setup(props, _ref) {
      let {
        slots
      } = _ref;
      const viewStates = new Map();
      const containerRef = vue.shallowRef(null);

      const setContainerRef = el => containerRef.value = el;

      const {
        monacoRef,
        unload
      } = useMonaco();
      const {
        editorRef
      } = useEditor(props, monacoRef, containerRef);
      const {
        disposeValidator
      } = useValidator(props, monacoRef, editorRef);
      const isEditorReady = vue.computed(() => !!monacoRef.value && !!editorRef.value);
      vue.onUnmounted(() => {
        var _disposeValidator$val, _editorRef$value, _editorRef$value$getM, _editorRef$value2;

        (_disposeValidator$val = disposeValidator.value) === null || _disposeValidator$val === void 0 ? void 0 : _disposeValidator$val.call(disposeValidator);
        (_editorRef$value = editorRef.value) === null || _editorRef$value === void 0 ? void 0 : (_editorRef$value$getM = _editorRef$value.getModel()) === null || _editorRef$value$getM === void 0 ? void 0 : _editorRef$value$getM.dispose();
        ((_editorRef$value2 = editorRef.value) === null || _editorRef$value2 === void 0 ? void 0 : _editorRef$value2.dispose()) ?? unload();
      }); // value

      vue.watch(() => props.value, newValue => {
        if (editorRef.value && editorRef.value.getValue() !== newValue) {
          editorRef.value.setValue(newValue);
        }
      }); // path

      vue.watch(() => props.path, (newPath, oldPath) => {
        const model = getOrCreateModel(monacoRef.value, props.value || props.defaultValue, props.language || props.defaultLanguage, newPath);

        if (model !== editorRef.value.getModel()) {
          props.saveViewState && viewStates.set(oldPath, editorRef.value.saveViewState());
          editorRef.value.setModel(model);
          props.saveViewState && editorRef.value.restoreViewState(viewStates.get(newPath));
        }
      }); // options

      vue.watch(() => props.options, options => editorRef.value && editorRef.value.updateOptions(options), {
        deep: true
      }); // theme

      vue.watch(() => props.theme, theme => monacoRef.value && monacoRef.value.editor.setTheme(theme)); // language

      vue.watch(() => props.language, language => isEditorReady.value && monacoRef.value.editor.setModelLanguage(editorRef.value.getModel(), language)); // line

      vue.watch(() => props.line, line => {
        // reason for undefined check: https://github.com/suren-atoyan/monaco-react/pull/188
        if (editorRef.value && !isUndefined(line)) {
          editorRef.value.revealLine(line);
        }
      });
      return () => {
        var _slots$default;

        return vue.createVNode(MonacoContainer, {
          "setContainerRef": setContainerRef,
          "width": props.width,
          "height": props.height,
          "isEditorReady": isEditorReady.value
        }, {
          default: () => [((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)) ?? 'loading...']
        });
      };
    }

  });

  function useEditor(props, monacoRef, containerRef) {
    const editorRef = vue.shallowRef(null);
    vue.onMounted(() => {
      const stop = vue.watch(monacoRef, () => {
        if (containerRef.value && monacoRef.value) {
          vue.nextTick(() => stop());
          createEditor();
        }
      }, {
        immediate: true
      });
    });

    function createEditor() {
      var _props$onBeforeMount, _editorRef$value3, _props$onMount;

      if (!containerRef.value || !monacoRef.value || editorRef.value) {
        return;
      } // editor before mount


      (_props$onBeforeMount = props.onBeforeMount) === null || _props$onBeforeMount === void 0 ? void 0 : _props$onBeforeMount.call(props, monacoRef.value);
      const autoCreatedModelPath = props.path || props.defaultPath;
      const defaultModel = getOrCreateModel(monacoRef.value, props.value || props.defaultValue, props.language || props.defaultLanguage, autoCreatedModelPath);
      editorRef.value = monacoRef.value.editor.create(containerRef.value, {
        model: defaultModel,
        theme: props.theme,
        automaticLayout: true,
        autoIndent: 'brackets',
        formatOnPaste: true,
        formatOnType: true,
        ...props.options
      }, props.overrideServices);
      (_editorRef$value3 = editorRef.value) === null || _editorRef$value3 === void 0 ? void 0 : _editorRef$value3.onDidChangeModelContent(event => {
        const value = editorRef.value.getValue();

        if (value !== props.value) {
          var _props$onUpdateValue, _props$onChange;

          (_props$onUpdateValue = props['onUpdate:value']) === null || _props$onUpdateValue === void 0 ? void 0 : _props$onUpdateValue.call(props, value);
          (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, value, event);
        }
      }); // editor mount

      (_props$onMount = props.onMount) === null || _props$onMount === void 0 ? void 0 : _props$onMount.call(props, editorRef.value, monacoRef.value);
    }

    return {
      editorRef
    };
  }

  function useValidator(props, monacoRef, editorRef) {
    const disposeValidator = vue.ref(null);
    const stop = vue.watch([monacoRef, editorRef], () => {
      if (monacoRef.value && editorRef.value) {
        vue.nextTick(() => stop());
        const changeMarkersListener = monacoRef.value.editor.onDidChangeMarkers(uris => {
          var _editorRef$value4, _editorRef$value4$get;

          const editorUri = (_editorRef$value4 = editorRef.value) === null || _editorRef$value4 === void 0 ? void 0 : (_editorRef$value4$get = _editorRef$value4.getModel()) === null || _editorRef$value4$get === void 0 ? void 0 : _editorRef$value4$get.uri;

          if (editorUri) {
            const currentEditorHasMarkerChanges = uris.find(uri => uri.path === editorUri.path);

            if (currentEditorHasMarkerChanges) {
              var _props$onValidate;

              const markers = monacoRef.value.editor.getModelMarkers({
                resource: editorUri
              });
              (_props$onValidate = props.onValidate) === null || _props$onValidate === void 0 ? void 0 : _props$onValidate.call(props, markers);
            }
          }
        });

        disposeValidator.value = () => changeMarkersListener === null || changeMarkersListener === void 0 ? void 0 : changeMarkersListener.dispose();
      }
    });
    return {
      disposeValidator
    };
  }

  Object.defineProperty(exports, 'loader', {
    enumerable: true,
    get: function () { return loader__default["default"]; }
  });
  exports.MonacoContainer = MonacoContainer;
  exports["default"] = Editor;
  exports.useMonaco = useMonaco;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
